/* Header file for leash.c, an AVR driver for Electronic Assembly's DOG series serial displays. 
Uses pins defined below and bit-bangs SPI data. Hardware SPI is selectable. It even features a frame buffer. 
Drawing functions originally written by Korken @ www.elektronikforumet.com.
Frame buffer contents are written to LCD with function LCD_update(). It might be a good idea to 
call this function on a timer interrupt so as to not have to keep track of when it's time to update. */


#include "avr/io.h"
#include "stdio.h"
#include "stdlib.h"
#include "util/delay.h"


#define DOGL				/* LCD type. Can be DOGS, DOGL, DOGXL, N3310 */
							/* Note that N3310 has a different LCD_init argument */
							/* where you specify (contrast, negative). */
							/* Contrast is in the range of 0xcc to 0xdd where 0xc8 is a good default. */

#define SPI_S				/* Define for software or hardware SPI. Can be SPI_S or SPI_H */
							/* Data direction registers have to be set regardless of S or H. */

#define LCD_X 128			/* LCD X resolution. */
#define LCD_Y 64			/* LCD Y resolution. */
#define CS_PORT PORTD		/* Port for chip select pin. */
#define CS_DDR DDRD 		/* Data direction register for chip select pin. */
#define CD_PORT PORTD		/* Port for command/data pin. */
#define SPI_PORT PORTD		/* Port for CLK and DATA pins. */
#define CD_DDR DDRD			/* Data direction register for command/data pin. */
#define RST_DDR DDRD		/* Data direction register for RST pi. */
#define SPI_DDR DDRD		/* Data direction register for CLK and DATA pins. */
#define RST_PORT PORTD		/* Port for RST pin. */
#define RST_PIN 0			/* Number of RST pin. */
#define CS_PIN 2			/* Number of CS pin. */
#define CD_PIN 4			/* Number of CD pin. */
#define DI 5				/* Number of data input pin (Not used for now). */
#define DO 3				/* Number of data output pin. */
#define CLK 5				/* Number of CLK pin. */


#define LCD_SELECT CS_PORT &= ~(1 << CS_PIN)		/* Macro for lcd selection. */
#define LCD_DESELECT CS_PORT |= (1 << CS_PIN)		/* Macro for lcd de-selection. */
#define LCD_SETDATA CD_PORT |= (1 << CD_PIN)		/* Macro for data output selection. */
#define LCD_CLEARDATA CD_PORT &= ~(1 << CD_PIN)		/* Macro for data output de-selection. */
#define SPI_SETDO SPI_PORT |= (1 << DO)				/* Macro for setting data pin. */
#define SPI_SETCLK	SPI_PORT |= (1<<CLK)			/* Macro for setting clock pin. */
#define SPI_CLEARDO SPI_PORT &= ~(1 << DO)			/* Macro for clearing data pin. */
#define SPI_CLEARCLK SPI_PORT &= ~(1 << CLK)		/* Macro for clearing clock pin. */

char fb[LCD_X][LCD_Y/8];	/* Frame buffer data array. */
unsigned char FB_xpos;		/* Frame buffer X position variable. */
unsigned char FB_ypos;		/* Frame buffer Y position variable. */


/* Function definitions */

void SPI_init(void);				/* Inits SPI control pins, note CLK is active low. */
void SPI_send(char data);			/* Sends a byte of data on SPI interface. */
void LCD_writecmd(char cmd);		/* Writes a command byte to LCD. */
void LCD_writedata(char data);		/* Writes a data byte to LCD. */
void LCD_setxy(char x, char y);		/* Sets X and Y positions on LCD. Note that Y denotes a page, not pixel. */
void FB_setxy(char x, char y);		/* Sets X and Y positions for a framebuffer write. Again, Y denotes a page, not pixel. */
void LCD_clear(void);				/* Clears LCD contents. */
void FB_clear(void);				/* Clears FB contents. */
void LCD_update(void);				/* Writes framebuffer contents to LCD. */
void FB_writechar(unsigned char x, unsigned char y, char ch);		/* Writes an alfanumeric character to framebuffer. */
void FB_writedata(char data);		/* Writes a byte to framebuffer and increases X position as LCD hardware would if being written a byte. */
void FB_drawpat(char pat, char count);	/* Draws a pattern in framebuffer. For instance pat=0b10101010 would set every other vertical pixel on that page. */
									/* Count denotes the number of bytes being written this way. */

#if defined(DOGS) || defined(DOGM) || defined(DOGL) || defined(DOGXL)
void LCD_init(void);
#endif

#ifdef N3310
void LCD_init(char contrast, char neg);
#endif

				/* Initializes LCD hardware, sets contrast and so on. You will need to edit the code for specific actions */
unsigned char FB_readdata(void);	/* Reads a byte from framebuffer */
void FB_drawdot(unsigned char x, unsigned char y);	/* Draws a dot anywhere in framebuffer */
void FB_cleardot(unsigned char x, unsigned char y);	/* Clears a dot anywhere in framebuffer */

void FB_drawline(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char clr);
					/* Draws a line from the specified cordinates (x1, y1) to the specified cordinates (x2, y2). */
					/* If clr is set to 1 it will not draw but instead clear all pixels along the way */

void FB_drawrect(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char fill, unsigned char clr);	
					/* Draws a box from the specified cordinates (x1, y1) to the specified cordinates (x2, y2). */
					/* If fill is set to 1 it will fill the box and if it is 0 it will just make the outline of a box. */
					/* If clr is set to 1 it will clear all pixels of the box and if it is 0 it will draw the box. */

void FB_drawcircle(unsigned char x1, unsigned char y1, unsigned char radius);
					/* Draws a circle at the specified cordinates (x, y) with the radius 'R'. */
void FB_writestring(unsigned char x, unsigned char y, char *s); /* writes sring to framebuffer */
void FB_clearrect(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2);
